@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of oRTP.
+ * This file is part of oRTP 
+ * (see https://gitlab.linphone.org/BC/public/ortp).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -31,8 +32,7 @@
 #define RTP_DEFAULT_MULTICAST_TTL 5	/*hops*/
 #define RTP_DEFAULT_MULTICAST_LOOPBACK 0  /*false*/
 #define RTP_DEFAULT_DSCP 0x00  /*best effort*/
-#define RTP_MAX_MIXER_TO_CLIENT_AUDIO_LEVEL 14 /* 14 because we use 1-byte headers */
-#define RTP_MAX_CLIENT_TO_MIXER_AUDIO_LEVEL 14 /* 14 because we use 1-byte headers */
+#define RTP_MAX_MIXER_TO_CLIENT_AUDIO_LEVEL 15 /* 15 because we can only put 15 csrc in a rtp packet */
 
 
 typedef struct rtp_header
@@ -91,6 +91,7 @@ typedef enum {
 	RTP_EXTENSION_MID = 1,
 	RTP_EXTENSION_CLIENT_TO_MIXER_AUDIO_LEVEL = 2,
 	RTP_EXTENSION_MIXER_TO_CLIENT_AUDIO_LEVEL = 3,
+	RTP_EXTENSION_FRAME_MARKING = 4,
 	RTP_EXTENSION_MAX = 15
 } rtp_extension_type_t;
 
@@ -99,6 +100,11 @@ typedef struct rtp_audio_level {
 	int dbov;
 } rtp_audio_level_t;
 
+#define RTP_FRAME_MARKER_START 			(1 << 7)
+#define RTP_FRAME_MARKER_END 			(1 << 6)
+#define RTP_FRAME_MARKER_INDEPENDENT 	(1 << 5)
+#define RTP_FRAME_MARKER_DISCARDABLE 	(1 << 4)
+
 #define RTP_TIMESTAMP_IS_NEWER_THAN(ts1, ts2) \
 	((uint32_t)((uint32_t)(ts1) - (uint32_t)(ts2)) < ((uint32_t)1 << 31))
 
@@ -118,6 +124,17 @@ extern "C"{
 #endif
 
 /* packet api */
+/* the first argument is a rtp_header_t  */
+#define rtp_header_set_seqnumber(hdr,seq)	(hdr)->seq_number=(htons(seq))
+#define rtp_header_set_timestamp(hdr,ts)	(hdr)->timestamp=(htonl(ts))
+#define rtp_header_set_ssrc(hdr,_ssrc)		(hdr)->ssrc=(htonl(_ssrc))
+ORTP_PUBLIC void rtp_header_add_csrc(rtp_header_t *hdr, uint32_t csrc);
+
+#define rtp_header_get_seqnumber(hdr)	(ntohs((hdr)->seq_number))
+#define rtp_header_get_timestamp(hdr)	(ntohl((hdr)->timestamp))
+#define rtp_header_get_ssrc(hdr)		(ntohl((hdr)->ssrc))
+#define rtp_header_get_csrc(hdr, idx)	(ntohl((hdr)->csrc[idx]))
+
 /* the first argument is a mblk_t. The header is supposed to be not splitted  */
 #define rtp_set_version(mp,value)	((rtp_header_t*)((mp)->b_rptr))->version=(value)
 #define rtp_set_padbit(mp,value)	((rtp_header_t*)((mp)->b_rptr))->padbit=(value)
@@ -125,21 +142,21 @@ extern "C"{
 #define rtp_set_cc(mp,value)	((rtp_header_t*)((mp)->b_rptr))->cc=(value)
 #define rtp_set_markbit(mp,value)	((rtp_header_t*)((mp)->b_rptr))->markbit=(value)
 #define rtp_set_payload_type(mp,pt)	((rtp_header_t*)((mp)->b_rptr))->paytype=(pt)
-#define rtp_set_seqnumber(mp,seq)	((rtp_header_t*)((mp)->b_rptr))->seq_number=(seq)
-#define rtp_set_timestamp(mp,ts)	((rtp_header_t*)((mp)->b_rptr))->timestamp=(ts)
-#define rtp_set_ssrc(mp,_ssrc)		((rtp_header_t*)((mp)->b_rptr))->ssrc=(_ssrc)
-ORTP_PUBLIC void rtp_add_csrc(mblk_t *mp ,uint32_t csrc);
+#define rtp_set_seqnumber(mp,seq)	rtp_header_set_seqnumber((rtp_header_t*)((mp)->b_rptr),(seq))
+#define rtp_set_timestamp(mp,ts)	rtp_header_set_timestamp((rtp_header_t*)((mp)->b_rptr),(ts))
+#define rtp_set_ssrc(mp,_ssrc)		rtp_header_set_ssrc((rtp_header_t*)((mp)->b_rptr),(_ssrc))
+ORTP_PUBLIC void rtp_add_csrc(mblk_t *mp, uint32_t csrc);
 
 #define rtp_get_version(mp)	(((rtp_header_t*)((mp)->b_rptr))->version)
 #define rtp_get_padbit(mp)	(((rtp_header_t*)((mp)->b_rptr))->padbit)
 #define rtp_get_markbit(mp)	(((rtp_header_t*)((mp)->b_rptr))->markbit)
 #define rtp_get_extbit(mp)	(((rtp_header_t*)((mp)->b_rptr))->extbit)
-#define rtp_get_timestamp(mp)	(((rtp_header_t*)((mp)->b_rptr))->timestamp)
-#define rtp_get_seqnumber(mp)	(((rtp_header_t*)((mp)->b_rptr))->seq_number)
+#define rtp_get_timestamp(mp)	rtp_header_get_timestamp((rtp_header_t*)((mp)->b_rptr))
+#define rtp_get_seqnumber(mp)	rtp_header_get_seqnumber((rtp_header_t*)((mp)->b_rptr))
 #define rtp_get_payload_type(mp)	(((rtp_header_t*)((mp)->b_rptr))->paytype)
-#define rtp_get_ssrc(mp)		(((rtp_header_t*)((mp)->b_rptr))->ssrc)
+#define rtp_get_ssrc(mp)		rtp_header_get_ssrc((rtp_header_t*)((mp)->b_rptr))
 #define rtp_get_cc(mp)		(((rtp_header_t*)((mp)->b_rptr))->cc)
-#define rtp_get_csrc(mp, idx)		(((rtp_header_t*)((mp)->b_rptr))->csrc[idx])
+#define rtp_get_csrc(mp, idx)	rtp_header_get_csrc((rtp_header_t*)((mp)->b_rptr),(idx))
 
 ORTP_PUBLIC int rtp_get_payload(mblk_t *packet, unsigned char **start);
 ORTP_PUBLIC int rtp_get_extheader(mblk_t *packet, uint16_t *profile, uint8_t **start_ext);
@@ -155,6 +172,10 @@ ORTP_PUBLIC int rtp_get_client_to_mixer_audio_level(mblk_t *packet, int id, bool
 ORTP_PUBLIC void rtp_add_mixer_to_client_audio_level(mblk_t *packet, int id, size_t size, rtp_audio_level_t *audio_levels);
 ORTP_PUBLIC int rtp_get_mixer_to_client_audio_level(mblk_t *packet, int id, rtp_audio_level_t* audio_levels);
 
+/* Frame marking api */
+ORTP_PUBLIC void rtp_add_frame_marker(mblk_t *packet, int id, uint8_t marker);
+ORTP_PUBLIC int rtp_get_frame_marker(mblk_t *packet, int id, uint8_t *marker);
+
 #ifdef __cplusplus
 }
 #endif
