@@ -1,22 +1,25 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of oRTP.
+ * This file is part of oRTP 
+ * (see https://gitlab.linphone.org/BC/public/ortp).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
-
+#ifdef HAVE_CONFIG_H
+#include "ortp-config.h"
+#endif
 #include "ortp/ortp.h"
 #include "utils.h"
 #include "ortp/rtpsession.h"
@@ -117,7 +120,7 @@ static void * outboud_simulator_thread(void *ctx){
 		sleep_until.tv_nsec=0;
 		rtp_session_schedule_outbound_network_simulator(session, &sleep_until);
 		if (sleep_until.tv_sec!=0) ortp_sleep_until(&sleep_until);
-		else ortp_sleep_ms(1);
+		else bctbx_sleep_ms(1);
 	}
 	return NULL;
 }
@@ -202,7 +205,7 @@ static mblk_t * simulate_latency(RtpSession *session, mblk_t *input){
 	struct timeval current;
 	mblk_t *output=NULL;
 	uint32_t current_ts;
-	ortp_gettimeofday(&current,NULL);
+	bctbx_gettimeofday(&current,NULL);
 	/*since we must store expiration date in reserved2(32bits) only(reserved1
 	already used), we need to reduce time stamp to milliseconds only*/
 	current_ts = 1000*current.tv_sec + current.tv_usec/1000;
@@ -226,13 +229,13 @@ static mblk_t * simulate_latency(RtpSession *session, mblk_t *input){
 }
 
 static int simulate_jitter_by_bit_budget_reduction(OrtpNetworkSimulatorCtx *sim, int budget_increase){
-	unsigned int r=ortp_random()%1000;
+	unsigned int r=bctbx_random()%1000;
 	float threshold,score;
 	int budget_adjust=0;
-	uint64_t now=ortp_get_cur_time_ms();
+	uint64_t now=bctbx_get_cur_time_ms();
 
 	if (sim->last_jitter_event==0){
-		sim->last_jitter_event=ortp_get_cur_time_ms();
+		sim->last_jitter_event=bctbx_get_cur_time_ms();
 	}
 
 	if (sim->in_jitter_event){
@@ -243,14 +246,14 @@ static int simulate_jitter_by_bit_budget_reduction(OrtpNetworkSimulatorCtx *sim,
 		threshold=500;
 	}
 	if (score>(int)threshold){
-		int64_t strength_rand=(int64_t)(sim->params.jitter_strength * (float)(ortp_random()%1000));
+		int64_t strength_rand=(int64_t)(sim->params.jitter_strength * (float)(bctbx_random()%1000));
 		sim->in_jitter_event=TRUE;
 		budget_adjust=(int)-((int64_t)budget_increase*strength_rand/1000LL);
 		/*ortp_message("jitter in progress... bit_budget_adjustement=%i, bit_budget=%i",budget_adjust,sim->bit_budget);*/
 	}else if (sim->in_jitter_event){
 		/*ortp_message("jitter ended.");*/
 		sim->in_jitter_event=FALSE;
-		sim->last_jitter_event=ortp_get_cur_time_ms();
+		sim->last_jitter_event=bctbx_get_cur_time_ms();
 	}
 	return budget_adjust;
 }
@@ -264,7 +267,7 @@ static mblk_t *simulate_bandwidth_limit_and_jitter(RtpSession *session, mblk_t *
 	mblk_t *output=NULL;
 	int overhead=(session->rtp.gs.sockfamily==AF_INET6) ? IP6_UDP_OVERHEAD : IP_UDP_OVERHEAD;
 
-	ortp_gettimeofday(&current,NULL);
+	bctbx_gettimeofday(&current,NULL);
 
 	if (sim->last_check.tv_sec==0){
 		sim->last_check=current;
@@ -321,7 +324,7 @@ static mblk_t *simulate_loss_rate(OrtpNetworkSimulatorCtx *net_sim_ctx, mblk_t *
 		loss_rate=net_sim_ctx->params.consecutive_loss_probability*1000.0f;
 	}
 
-	rrate = ortp_random() % 1000;
+	rrate = bctbx_random() % 1000;
 
 	if (rrate >= loss_rate) {
 		if (net_sim_ctx->consecutive_drops){
@@ -476,7 +479,7 @@ static void rtp_session_schedule_outbound_network_simulator(RtpSession *session,
 				todrop = om; /*simulate a packet loss, only RTP packets can be dropped. Timestamp is not set for RTCP packets*/
 			}else if (ortp_timespec_compare(&packet_time, &current) <= 0){
 				/*it is time to send this packet*/
-				
+
 				_ortp_sendto(is_rtp_packet ? session->rtp.gs.socket : session->rtcp.gs.socket, om, 0, (struct sockaddr*)&om->net_addr, om->net_addrlen);
 				todrop = om;
 			}else {
@@ -498,4 +501,3 @@ static void rtp_session_schedule_outbound_network_simulator(RtpSession *session,
 		}
 	}
 }
-
