@@ -1,19 +1,20 @@
 /*
- * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ * Copyright (c) 2010-2022 Belledonne Communications SARL.
  *
- * This file is part of oRTP.
+ * This file is part of oRTP 
+ * (see https://gitlab.linphone.org/BC/public/ortp).
  *
  * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of the GNU Affero General Public License as
+ * published by the Free Software Foundation, either version 3 of the
+ * License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Affero General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
@@ -41,7 +42,7 @@ static uint32_t posix_timer_time=0;		/*in milisecond */
 void posix_timer_init(void)
 {
 	posix_timer.state=RTP_TIMER_RUNNING;
-	ortp_gettimeofday(&orig,NULL);
+	bctbx_gettimeofday(&orig,NULL);
 	posix_timer_time=0;
 }
 
@@ -52,7 +53,7 @@ void posix_timer_do(void)
 {
 	int diff,time;
 	struct timeval tv;
-	ortp_gettimeofday(&cur,NULL);
+	bctbx_gettimeofday(&cur,NULL);
 	time=((cur.tv_usec-orig.tv_usec)/1000 ) + ((cur.tv_sec-orig.tv_sec)*1000 );
 	if ( (diff=time-posix_timer_time)>50){
 		ortp_warning("Must catchup %i miliseconds.",diff);
@@ -67,11 +68,11 @@ void posix_timer_do(void)
 #else
 		select(0,NULL,NULL,NULL,&tv);
 #endif
-		ortp_gettimeofday(&cur,NULL);
+		bctbx_gettimeofday(&cur,NULL);
 		time=((cur.tv_usec-orig.tv_usec)/1000 ) + ((cur.tv_sec-orig.tv_sec)*1000 );
 	}
 	posix_timer_time+=POSIXTIMER_INTERVAL/1000;
-	
+
 }
 
 void posix_timer_uninit(void)
@@ -84,8 +85,8 @@ RtpTimer posix_timer={	0,
 						posix_timer_do,
 						posix_timer_uninit,
 						{0,POSIXTIMER_INTERVAL}};
-							
-							
+
+
 #else //_WIN32
 
 
@@ -294,7 +295,7 @@ void win_timer_do(void)
 
 void win_timer_close(void)
 {
-	timeKillEvent(timerId); 
+	timeKillEvent(timerId);
 }
 
 RtpTimer toto;
